library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity cc1000_v3_0 is
	generic (
		-- Users to add parameters here
		DBG_LEN        : integer :=0;	
        DATA_BYTES_MAX : integer                   := 64;  --max no. of data bytes; 4*n, where n=1,2,...
        PREAMBLE_BITS  : integer                   := 96;  --no. of preamble bits for tx,  must be EVEN
        SFD_BYTES      : std_logic_vector(0 to 15) := X"E39B";  -- Start of Frame Delimiter		
        CLK_DIV_ADC    : integer := 64; -- Clock of ADC CLK_ACD(<800kHz) = CLK/CLK_DIV_ADC
        CLK_DIV_CC1000 : integer := 3;  -- Config clock PCLK(<10MHz) of CC1000 f(PCLK) = CLK/(2*CLK_DIV_CC1000)        
		-- User parameters ends
		-- Do not modify the parameters beyond this line


		-- Parameters of Axi Slave Bus Interface S00_AXI
		C_S00_AXI_DATA_WIDTH	: integer	:= 32;
		C_S00_AXI_ADDR_WIDTH	: integer	:= 4
	);
	port (
		-- Users to add ports here
	    dbg_o            : out std_logic_vector(DBG_LEN-1 downto 0);
        CC1000_DCLK_i       : in    std_logic;    -- to CC1000
        CC1000_DIO_io       : inout std_logic;    -- to CC1000
        CC1000_PCLK_o       : out   std_logic;    -- to CC1000
        CC1000_PDATA_io     : inout std_logic;    -- to CC1000
        CC1000_PALE_o       : out   std_logic;    -- to CC1000
		
        MCP3201_clk_o  : out   std_logic;
        MCP3201_data_i : in    std_logic;
        MCP3201_cs_o   : out   std_logic;                

		-- User ports ends
		-- Do not modify the ports beyond this line


		-- Ports of Axi Slave Bus Interface S00_AXI
		s00_axi_aclk	: in std_logic;
		s00_axi_aresetn	: in std_logic;
		s00_axi_awaddr	: in std_logic_vector(C_S00_AXI_ADDR_WIDTH-1 downto 0);
		s00_axi_awprot	: in std_logic_vector(2 downto 0);
		s00_axi_awvalid	: in std_logic;
		s00_axi_awready	: out std_logic;
		s00_axi_wdata	: in std_logic_vector(C_S00_AXI_DATA_WIDTH-1 downto 0);
		s00_axi_wstrb	: in std_logic_vector((C_S00_AXI_DATA_WIDTH/8)-1 downto 0);
		s00_axi_wvalid	: in std_logic;
		s00_axi_wready	: out std_logic;
		s00_axi_bresp	: out std_logic_vector(1 downto 0);
		s00_axi_bvalid	: out std_logic;
		s00_axi_bready	: in std_logic;
		s00_axi_araddr	: in std_logic_vector(C_S00_AXI_ADDR_WIDTH-1 downto 0);
		s00_axi_arprot	: in std_logic_vector(2 downto 0);
		s00_axi_arvalid	: in std_logic;
		s00_axi_arready	: out std_logic;
		s00_axi_rdata	: out std_logic_vector(C_S00_AXI_DATA_WIDTH-1 downto 0);
		s00_axi_rresp	: out std_logic_vector(1 downto 0);
		s00_axi_rvalid	: out std_logic;
		s00_axi_rready	: in std_logic
	);
end cc1000_v3_0;

architecture arch_imp of cc1000_v3_0 is

	-- component declaration
	component cc1000_v3_0_S00_AXI is
		generic (
        DBG_LEN        :    integer :=128;
		DATA_BYTES_MAX : integer                   := 64;  --max no. of data bytes; 4*n, where n=1,2,...
        PREAMBLE_BITS  : integer                   := 96;  --no. of preamble bits for tx,  must be EVEN
        SFD_BYTES      : std_logic_vector(0 to 15) := X"E39B";  -- Start of Frame Delimiter		
        CLK_DIV_ADC    : integer := 64; -- Clock of ADC CLK_ACD(<800kHz) = CLK/CLK_DIV_ADC
        CLK_DIV_CC1000 : integer := 3;  -- Config clock PCLK(<10MHz) of CC1000 f(PCLK) = CLK/(2*CLK_DIV_CC1000)        

		C_S_AXI_DATA_WIDTH	: integer	:= 32;
		C_S_AXI_ADDR_WIDTH	: integer	:= 4
		);
		port (
        dbg_o            : out std_logic_vector(DBG_LEN-1 downto 0);
        CC1000_DCLK_i       : in    std_logic;    -- to CC1000
        CC1000_DIO_io       : inout std_logic;    -- to CC1000
        CC1000_PCLK_o       : out   std_logic;    -- to CC1000
        CC1000_PDATA_io     : inout std_logic;    -- to CC1000
        CC1000_PALE_o       : out   std_logic;    -- to CC1000
		
        MCP3201_clk_o  : out   std_logic;
        MCP3201_data_i : in    std_logic;
        MCP3201_cs_o   : out   std_logic;                		

		S_AXI_ACLK	: in std_logic;
		S_AXI_ARESETN	: in std_logic;
		S_AXI_AWADDR	: in std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
		S_AXI_AWPROT	: in std_logic_vector(2 downto 0);
		S_AXI_AWVALID	: in std_logic;
		S_AXI_AWREADY	: out std_logic;
		S_AXI_WDATA	: in std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
		S_AXI_WSTRB	: in std_logic_vector((C_S_AXI_DATA_WIDTH/8)-1 downto 0);
		S_AXI_WVALID	: in std_logic;
		S_AXI_WREADY	: out std_logic;
		S_AXI_BRESP	: out std_logic_vector(1 downto 0);
		S_AXI_BVALID	: out std_logic;
		S_AXI_BREADY	: in std_logic;
		S_AXI_ARADDR	: in std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
		S_AXI_ARPROT	: in std_logic_vector(2 downto 0);
		S_AXI_ARVALID	: in std_logic;
		S_AXI_ARREADY	: out std_logic;
		S_AXI_RDATA	: out std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
		S_AXI_RRESP	: out std_logic_vector(1 downto 0);
		S_AXI_RVALID	: out std_logic;
		S_AXI_RREADY	: in std_logic
		);
	end component cc1000_v3_0_S00_AXI;

begin

-- Instantiation of Axi Bus Interface S00_AXI
cc1000_v3_0_S00_AXI_inst : cc1000_v3_0_S00_AXI
	generic map (
        DBG_LEN             => DBG_LEN,
	    DATA_BYTES_MAX      => DATA_BYTES_MAX,
        PREAMBLE_BITS       => PREAMBLE_BITS,
        SFD_BYTES           => SFD_BYTES,
	    CLK_DIV_ADC         => CLK_DIV_ADC,
        CLK_DIV_CC1000      => CLK_DIV_CC1000,

		C_S_AXI_DATA_WIDTH	=> C_S00_AXI_DATA_WIDTH,
		C_S_AXI_ADDR_WIDTH	=> C_S00_AXI_ADDR_WIDTH
	)
	port map (
        dbg_o => dbg_o,
	    
        CC1000_DCLK_i       => CC1000_DCLK_i,
        CC1000_DIO_io       => CC1000_DIO_io,
        CC1000_PCLK_o       => CC1000_PCLK_o,
        CC1000_PDATA_io     => CC1000_PDATA_io, 
        CC1000_PALE_o       => CC1000_PALE_o,

	    MCP3201_clk_o => MCP3201_clk_o,
        MCP3201_data_i => MCP3201_data_i,
        MCP3201_cs_o   => MCP3201_cs_o,

		S_AXI_ACLK	=> s00_axi_aclk,
		S_AXI_ARESETN	=> s00_axi_aresetn,
		S_AXI_AWADDR	=> s00_axi_awaddr,
		S_AXI_AWPROT	=> s00_axi_awprot,
		S_AXI_AWVALID	=> s00_axi_awvalid,
		S_AXI_AWREADY	=> s00_axi_awready,
		S_AXI_WDATA	=> s00_axi_wdata,
		S_AXI_WSTRB	=> s00_axi_wstrb,
		S_AXI_WVALID	=> s00_axi_wvalid,
		S_AXI_WREADY	=> s00_axi_wready,
		S_AXI_BRESP	=> s00_axi_bresp,
		S_AXI_BVALID	=> s00_axi_bvalid,
		S_AXI_BREADY	=> s00_axi_bready,
		S_AXI_ARADDR	=> s00_axi_araddr,
		S_AXI_ARPROT	=> s00_axi_arprot,
		S_AXI_ARVALID	=> s00_axi_arvalid,
		S_AXI_ARREADY	=> s00_axi_arready,
		S_AXI_RDATA	=> s00_axi_rdata,
		S_AXI_RRESP	=> s00_axi_rresp,
		S_AXI_RVALID	=> s00_axi_rvalid,
		S_AXI_RREADY	=> s00_axi_rready
	);

	-- Add user logic here

	-- User logic ends

end arch_imp;
