-- Modu? konfiguruj?cy CC1000. Napisany w ramach pracy magisterskiej na WETI PG.
--
-- Autor: Bartosz Marcin Durski 
--
-- Opis:
-- Linia 'busy' musi mie stan 0. Linie 'start', 'wr', 'data_i' i 'addr' ustalaj? si,
-- przy czym linia 'start' przechodzi z 0 na 1. Stan 1 musi trwa maks. 15*CLK_DIV_VAL
-- cykli zegara. Wraz z przejciem 'start' w stan 1, linia 'busy' przechodzi na 1, a? do
-- zakoczenia przesy?ania/odbierania danych.
--
-- ** = sygna? wychodzi z lub wchodzi do uk?adu

library ieee;
use ieee.std_logic_1164.all;

library UNISIM;
use UNISIM.VCOMPONENTS.ALL;

entity user_component_cc1000_config is
	generic (
		CLK_DIV_VAL	: 	integer := 3	-- dzielnik zegara g?wnego dla pclk; PCLK = CLK/(2*CLK_DIV_VAL)
	);
	port(
		-- wejcia do modu?u od strony uk?adu
		clk_i		: in	std_logic;						-- zegar magistrali
		reset_i	: in	std_logic;						-- reset asynchroniczny
		
		start_i	: in	std_logic;						-- sygnalizuje rozpoczcie zappisu/odczytu
		wr_i		: in	std_logic;						-- czy ZAPIS czy ODCZYT
		data_i	: in	std_logic_vector(7 downto 0);	-- dane do wys?ania
		addr_i	: in	std_logic_vector(6 downto 0);	-- adres rejestru
		
		-- wejcia do i wyjcia z modu?u od strony wiata
		pdata_io	: inout	std_logic;						-- ** linia danych
		pale_o	: out	std_logic;						-- ** linia kontroluj?ca
		pclk_o	: out	std_logic;						-- ** zegar taktuj?cy przesy? danych do/z przetwornika; fmax=10MHz
		
		-- wyjcia z modu?u od strony uk?adu
		data_o	: out	std_logic_vector(7 downto 0);	-- rejestr odbiorczy - aktualizowany po odebraniu ca?ej danej
		addr_o	: out	std_logic_vector(6 downto 0);	-- adres rejestru z ktrego czytamy - aktualizowany po odebraniu ca?ej danej
		busy_o	: out	std_logic;						-- sygnalizuje zajto
		
		dbg_pdata_o : out std_logic
	);
end entity user_component_cc1000_config;

architecture user_component_cc1000_config_arch of user_component_cc1000_config is
component IOBUF is
port (
	I	: in  std_logic;
	IO	: inout std_logic;
	O	: out std_logic;
	T	: in  std_logic
);
end component;

signal request_start		: std_logic := '0';					-- informacja, ?e by?o ??danie startu
signal reset_request_start	: std_logic	:= '0';					-- usunicie informacji, ?e by?o ??danie startu

signal pclk_tmp				: std_logic := '1';					-- sygna? zegarowy taktuj?cy przesy? danych do/z przetwornika
signal pale_tmp				: std_logic := '1';					-- to?samy z 'pale'

signal busy_tmp				: std_logic := '0';					-- to?samy z 'busy'

signal data_to_send			: std_logic_vector(15 downto 0);	-- dane do wys?ania (po??czenie 'addr_i', 'wr', 'data_i')

signal tmp_data_o			: std_logic_vector(7 downto 0);		-- odbiorczy rejestr tymczasowy; to on aktualizuje wyjcie data_o

signal tri_state			: std_logic := '1';					-- prze??cza bufor trjstanowy '0'=out , '1'=Hi-Z
signal tri_out				: std_logic;
signal tri_in				: std_logic;

-- IDLE				- stan ja?owy
-- PALE_LOW			- ustawia 'pale' na 0
-- PALE_HIGH		- ustawia 'pale' na 1
-- PDATA_TO_HI_Z	- prze?acza 'pdata' w stan Hi-Z
-- PCLK_LOW			- zmienia stan 'pclk' na 0
-- PCLK_HIGH		- zmienia stan 'pclk' na 1
type states is (IDLE, PALE_LOW, PALE_HIGH, PDATA_TO_HI_Z, PCLK_LOW, PCLK_HIGH);
signal state		: states := IDLE;		-- opisuje stany w jakich znajduje si konfigurator

signal test_vec				: std_logic_vector(0 to 7) := X"01";
signal test_error			: std_logic := '0';

begin
	-- Bufor 3-stanowy
	iobuf_0 : IOBUF
    port map (
      I		=> tri_out,
      IO	=> pdata_io,
      O		=> tri_in,
      T		=> tri_state
    );

----	TEST1: Do testowania odbioru danych przez RX_TX. Zakomentowa wszystko poni?ej tego komentarza
----	Test ma pokaza, czy odbiornik dobrze odbiera
----	DIO odbiornika po??czy z PALE nadajnika
----	DCLK odbiornika po??czy z PCLK nadajnika
--	pale	<= pale_tmp;
--	pclk	<= pclk_tmp;
--	process(clk)
--		variable timer : integer range 0 to 1023 := 0;
--	begin
--		if rising_edge(clk) then
--			if timer = 1023 then
--				timer := 0;
--				pclk_tmp	<= not pclk_tmp;
--			else
--				timer := timer + 1;
--			end if;
--		end if;
--	end process;
--	
--	process(pclk_tmp)
--		variable timer : integer range 0 to 32767 := 0;
--	begin
--		if falling_edge(pclk_tmp) then
--			if timer = 32767 then
--				timer := 0;
--			else
--				if timer = 0 then
--					test_vec(0)		<= test_vec(1);
--					test_vec(1)		<= test_vec(2);
--					test_vec(2)		<= test_vec(3);
--					test_vec(3)		<= test_vec(4);
--					test_vec(4)		<= test_vec(5);
--					test_vec(5)		<= test_vec(6);
--					test_vec(6)		<= test_vec(7);
--					test_vec(7)		<= test_vec(0);
--					pale_tmp 		<= '0';
--				
--				elsif timer < 101 then pale_tmp	<= not pale_tmp;
--				
--				elsif timer = 101 then pale_tmp	<= '1';	-- SFD(0)
--				elsif timer = 102 then pale_tmp	<= '1';	-- SFD(1)
--				elsif timer = 103 then pale_tmp	<= '1';	-- SFD(2)
--				elsif timer = 104 then pale_tmp	<= '0';	-- SFD(3)
--				elsif timer = 105 then pale_tmp	<= '0';	-- SFD(4)
--				elsif timer = 106 then pale_tmp	<= '0';	-- SFD(5)
--				elsif timer = 107 then pale_tmp	<= '1';	-- SFD(6)
--				elsif timer = 108 then pale_tmp	<= '1';	-- SFD(7)
--					
--				elsif timer = 109 then pale_tmp	<= '1';	-- SFD(8)
--				elsif timer = 110 then pale_tmp	<= '0';	-- SFD(9)
--				elsif timer = 111 then pale_tmp	<= '0';	-- SFD(10)
--				elsif timer = 112 then pale_tmp	<= '1';	-- SFD(11)
--				elsif timer = 113 then pale_tmp	<= '1';	-- SFD(12)
--				elsif timer = 114 then pale_tmp	<= '0';	-- SFD(13)
--				elsif timer = 115 then pale_tmp	<= '1';	-- SFD(14)
--				elsif timer = 116 then pale_tmp	<= '1';	-- SFD(15)
--					
--				elsif timer = 117 then pale_tmp	<= '0';	-- DATA(0)
--				elsif timer = 118 then pale_tmp	<= '0';	-- DATA(1)
--				elsif timer = 119 then pale_tmp	<= '0';	-- DATA(2)
--				elsif timer = 120 then pale_tmp	<= test_error; test_error <= not test_error;	-- DATA(3) 
--				elsif timer = 121 then pale_tmp	<= '0';	-- DATA(4)
--				elsif timer = 122 then pale_tmp	<= '0';	-- DATA(5)
--				elsif timer = 123 then pale_tmp	<= '1';	-- DATA(6)
--				elsif timer = 124 then pale_tmp	<= '0';	-- DATA(7)
--					
--				elsif timer = 125 then pale_tmp	<= test_vec(0);	-- DATA(8)
--				elsif timer = 126 then pale_tmp	<= test_vec(1);	-- DATA(9)
--				elsif timer = 127 then pale_tmp	<= test_vec(2);	-- DATA(10)
--				elsif timer = 128 then pale_tmp	<= test_vec(3);	-- DATA(11)
--				elsif timer = 129 then pale_tmp	<= test_vec(4);	-- DATA(12)
--				elsif timer = 130 then pale_tmp	<= test_vec(5);	-- DATA(13)
--				elsif timer = 131 then pale_tmp	<= test_vec(6);	-- DATA(14)
--				elsif timer = 132 then pale_tmp	<= test_vec(7);	-- DATA(15)
--				
--				else pale_tmp <= '0';
--				end if;
--				timer := timer + 1;
--			end if;
--		end if;
--	end process;

	pclk_o	<= pclk_tmp;
	pale_o	<= pale_tmp;
	busy_o	<= busy_tmp or request_start;
	
	data_to_send	<= (addr_i & wr_i & data_i);


	-- Zapisuje/odczytuje dane do/z rejestrw konfiguracyjnych CC1000
	WRITE_READ_PROC : process(clk_i, reset_i)
		variable counter	: integer range 0 to 15 := 15;							-- wskazuje kolejny bit do wys?ania
		variable timer		: integer range 0 to CLK_DIV_VAL-1 := CLK_DIV_VAL-1;	-- odlicza zmiany stanu zegara pclk; PLCK = CLK/(2*CLK_DIV_VAL)
	begin
		if rising_edge(clk_i) then
			if reset_i = '1' then
				state				<= IDLE;
				tri_state			<= '1';
				pale_tmp			<= '1';
				pclk_tmp			<= '1';
				data_o				<= (others => '0');
				addr_o				<= (others => '0');
				busy_tmp			<= '0';
				reset_request_start	<= '0';
				counter				:= 15;
				timer				:= CLK_DIV_VAL-1;
			else
				if request_start = '1' and busy_tmp = '0' then
					busy_tmp			<= '1';
					reset_request_start	<= '1';
					state				<= PALE_LOW;
					counter				:= 15;
					timer				:= CLK_DIV_VAL-1;
				else
					reset_request_start	<= '0';
					
					if timer = 0 or busy_tmp = '0' then
						timer	:= CLK_DIV_VAL-1;
						
						case state is
							-- stan nic nie robienia - oczekiwania na now? prob zapisu/odczytu
							when IDLE =>
								tri_state	<= '1';
								pale_tmp	<= '1';
								pclk_tmp	<= '1';
								busy_tmp	<= '0';
								counter		:= 15;
							
							-- przejcie 'pale' w stan 0
							when PALE_LOW =>
								pale_tmp	<= '0';
								state		<= PCLK_HIGH;
							
							-- przejcie 'pale' w stan 1
							when PALE_HIGH =>
								pale_tmp	<= '1';
								if wr_i = '1' then
									tri_state	<= '0';
									tri_out		<= data_to_send(counter);
								end if;
								state	<= PCLK_LOW;
							
							when PCLK_HIGH =>
								pclk_tmp	<= '1';
								if wr_i = '1' then
									if counter = 7 then
										state	<= PALE_HIGH;	
									else
										state		<= PCLK_LOW;
										tri_state	<= '0';
										tri_out		<= data_to_send(counter);
									end if;
								else
									-- gdy jest tryb odczytu, to nastpny stan ma za zadanie prze??czy 'pdata' w stan Hi-Z
									if counter = 7 and pale_tmp = '0' then
										state	<= PDATA_TO_HI_Z;
									else
										state	<= PCLK_LOW;
										if counter > 7 then
											tri_state	<= '0';
											tri_out		<= data_to_send(counter);
										else
											counter := counter - 1;
										end if;
									end if;
								end if;
							
							when PCLK_LOW =>
								pclk_tmp	<= '0';
								if wr_i = '1' then
									if counter = 0 then
										state	<= IDLE;
									else
										state	<= PCLK_HIGH;
										counter	:= counter - 1;
									end if;
								else
									if counter = 0 then
										state				<= IDLE;
										tmp_data_o(counter)	<= tri_in;
										data_o(7 downto 1)	<= tmp_data_o(7 downto 1);
										data_o(0)			<= tri_in;
										addr_o				<= addr_i;
									else
										state	<= PCLK_HIGH;
										if counter < 8 then
											tmp_data_o(counter)	<= tri_in;
										else
											counter	:= counter - 1;
										end if;
									end if;
								end if;
								
							when PDATA_TO_HI_Z =>
								tri_state	<= '1';
								state		<= PALE_HIGH;
						end case;
					else
						timer := timer - 1;
					end if;
				end if;
			end if;
		end if;
	end process WRITE_READ_PROC;


	-- Sterowanie sygna?em 'request_start', ktry wyzwala zapis/odczyt
	REQUEST_PROC : process(reset_i, start_i, reset_request_start, busy_tmp)
	begin
		if reset_i = '1' or reset_request_start = '1' then
			request_start	<= '0';
		elsif start_i = '1' and busy_tmp = '0' then
			request_start	<= '1';
		else
			request_start	<= request_start;
		end if;
	end process REQUEST_PROC;
	
	
	dbg_pdata_o <= tri_out;
end architecture user_component_cc1000_config_arch;
