--	Modu?	obs?ugi	przetwornika	A/C.	Napisany	w	ramach	pracy	magisterskiej	na	WETI	PG.
--
--	Autor:	Bartosz	Marcin	Durski	
--
--	Opis:
--	Uruchomienie	przetwarzania	nastpuje	na	2	sposoby:
--	a)	??danie	od	odbiornika	(wy?szy	priorytet)
--	b)	??danie	od	programu	(ni?szy	priorytet)
--	Priorytet	ma	to	znaczenie,	?e	gdy	pojawi	si	??danie	o	wy?szym	priorytecie,	to	przetwarzanie	wynik?e	z	??dania
--	o	ni?szym	priorytecie	zostaje	przerwane	i	na	nowo	jest	rozpoczynane.	Impuls	??dania	powinien	trwa	jak	najkrcej
--	(najlepiej	1	cykl	zegara).	Wynik	mo?na	odczyta	z	rejestru	wyjciowego,	ktry	jest	nadpisywany	po	wykonaniu	si
--	ca?ego	przetwarzania.	Odczytuj?c	dane	nale?y	sprawdza	stan	linii	'busy'.	Je?eli	jest	on	1,	to	znaczy,	?e
--	przetwarzanie	jeszcze	si	nie	zakoczy?o	i	rejestr	wyjciowy	nie	zosta?	jeszcze	zaktualizowany.
--
--	**	=	sygna?	wychodzi	z	lub	wchodzi	do	uk?adu

library	ieee;
use	ieee.std_logic_1164.all;
use	ieee.std_logic_unsigned.all;

entity	user_component_mcp3201	is
	generic(
		CLK_DIV_VAL	:	integer	:=	64		--	dzielnik	zegara	g?wnego;	CLK_OUT	=	CLK/CLK_DIV_VAL	;	minimum	2
	);
	port(
		--	wejcia	do	modu?u	od	strony	uk?adu
		clk_i					:	in	std_logic;						--	zegar	magistrali
		reset_i					:	in	std_logic;						--	reset	asynchroniczny
		
		request_from_rx_i		:	in	std_logic;						--	??danie	przetworzenia	od	odbiornika
		request_from_prog_i		:	in	std_logic;						--	??danie	przetworzenia	od	programu
		
		--	wejcia	do	i	wyjcia	z	modu?u	od	strony	wiata
		MCP3201_din_i					:	in	std_logic;						--	**	szeregowe	wyjcie	danych	przetwornika
		MCP3201_clk_o					:	out	std_logic;						--	**	zegar	taktuj?cy	komunikacj	z	przetwornikiem;	fmax=800kHz
		MCP3201_cs_o					:	out	std_logic;						--	**	aktywacja	przetwornika	(active	low)
		
		--	wyjcia	z	modu?u	od	strony	uk?adu
		adc_busy_o					:	out	std_logic;						--	przetwornik	jest	zajty	przetwarzaniem
		adc_dout_o					:	out	std_logic_vector(11	downto	0);	--	rejestr	wyjciowy	z	dan?	do	odczytu
        
        adc_state_dbg_o: out std_logic_vector(3 downto 0)
		
	);
end	entity;


architecture	user_component_mcp3201_arch	of	user_component_mcp3201	is
    type state_t is (INIT, CS_LOW, SAMPLING1, SAMPLING2, B11, B10, B9, B8, B7, B6, B5, B4, B3, B2, B1, B0);    
    signal	clk700kHz    : std_logic;
    signal state         : state_t;
    signal state_next    : state_t;
    signal start         : std_logic;
    signal start_ack     : std_logic;
    signal restart       : std_logic;
    signal MCP3201_cs    : std_logic;

begin
	MCP3201_clk_o	<=	clk700kHz;
	
	process (state)
	begin
        adc_state_dbg_o <= (others => '0');
        case state is
            when INIT      => adc_state_dbg_o <="0000";
            when CS_LOW    => adc_state_dbg_o <="0001";
            when SAMPLING1 => adc_state_dbg_o <="0010";
            when SAMPLING2 => adc_state_dbg_o <="0011";
            when  B11      => adc_state_dbg_o <="0100";
            when B10       => adc_state_dbg_o <="0101";
            when B9        => adc_state_dbg_o <="0110";
            when B8        => adc_state_dbg_o <="0111";
            when B7        => adc_state_dbg_o <="1000";
            when B6        => adc_state_dbg_o <="1001";
            when B5        => adc_state_dbg_o <="1010";
            when B4        => adc_state_dbg_o <="1011";
            when B3        => adc_state_dbg_o <="1100";
            when B2        => adc_state_dbg_o <="1101";
            when B1        => adc_state_dbg_o <="1110";
            when B0        => adc_state_dbg_o <="1111";
        end case;
	end process;
	
	process (state)
	begin 
	   adc_busy_o <= '0';
	   if state /= INIT or start = '1' or restart = '1' then
	       adc_busy_o <= '1';
       end if; 
	end process;		

	START_REQ: process (reset_i, clk_i)
    begin
        if	reset_i	=	'1'	then
            start <= '0';
        elsif rising_edge(clk_i) then
            if start_ack = '1' then
                start   <= '0';        
            end if;
            if request_from_rx_i = '1' or (request_from_prog_i = '1' and state = INIT) then
                start <= '1';
            end if;                        
        end if;
    end process;
    
    
    MCP_CS_SEQ: process (reset_i, clk700kHz)
	begin
        if reset_i = '1' then
           MCP3201_cs_o <= '1';            
        elsif falling_edge(clk700kHz) then            
           MCP3201_cs_o <= MCP3201_cs;
       end if;      
    end process;
	
	STATE_MACHINE_SEQ: process (reset_i, clk700kHz)
	begin
        if reset_i = '1' then
            state <= INIT;
            adc_dout_o <= (others => '0');
        elsif rising_edge(clk700kHz) then            
            state <= state_next;                        
            case state is                
                when B11 => adc_dout_o(11) <= MCP3201_din_i;   
                when B10 => adc_dout_o(10) <= MCP3201_din_i;
                when B9  => adc_dout_o(9)  <= MCP3201_din_i;
                when B8  => adc_dout_o(8)  <= MCP3201_din_i;
                when B7  => adc_dout_o(7)  <= MCP3201_din_i;
                when B6  => adc_dout_o(6)  <= MCP3201_din_i;
                when B5  => adc_dout_o(5)  <= MCP3201_din_i;
                when B4  => adc_dout_o(4)  <= MCP3201_din_i;
                when B3  => adc_dout_o(3)  <= MCP3201_din_i;
                when B2  => adc_dout_o(2)  <= MCP3201_din_i;
                when B1  => adc_dout_o(1)  <= MCP3201_din_i;
                when B0  => adc_dout_o(0)  <= MCP3201_din_i;
                when others => null;
            end case;            
	   end if;    
	end process;
	
	STATE_MACHINE_COMB: process (state, start) 
	begin
	   start_ack <= '0';
	   MCP3201_cs <= '0';
       
	   case state is 
           when INIT =>	       
               MCP3201_cs <= '1';
               state_next <= INIT;
               if start = '1' then
                    state_next <= CS_LOW;
                end if;
           when CS_LOW =>            
                start_ack <= '1';
                state_next <= SAMPLING1;
           when SAMPLING1 =>            
                state_next <= SAMPLING2;
           when SAMPLING2 =>            
                state_next <= B11;
           when B11 =>            
                state_next <= B10;
           when B10 =>            
                state_next <= B9;
           when B9 =>            
                state_next <= B8;
           when B8 =>            
                state_next <= B7;
           when B7 =>
                state_next <= B6;
           when B6 =>
                state_next <= B5;
           when B5 =>
                state_next <= B4;
           when B4 =>
                state_next <= B3;
           when B3 =>
                state_next <= B2;
           when B2 =>
                state_next <= B1;
           when B1 =>
                state_next <= B0;
           when B0 =>
                state_next <= INIT;                
        end case;
        
        if state /= INIT and start = '1' then
            state_next <= INIT;
            MCP3201_cs <= '1';
        end if;                	   
	end process;
	

	--	Dzielnik	zegara	CLK_OUT	=	CLK/CLK_DIV_VAL
	ADC_CLK_PROC	:	process(clk_i)	is
		variable	counter_v	:	integer	range	0	to	CLK_DIV_VAL-1	:=	0;
	begin
        if	reset_i	=	'1'	then
            counter_v	:= 0;
            clk700kHz	<=	'0';
        elsif	rising_edge(clk_i)	then
			if	counter_v	<	CLK_DIV_VAL/2	then
				clk700kHz	<=	'0';
			else
				clk700kHz	<=	'1';
			end	if;
			if	counter_v	=	CLK_DIV_VAL-1	then
				counter_v	:=	0;
			else
				counter_v	:=	counter_v	+	1;
			end	if;
		end	if;
	end	process	ADC_CLK_PROC;
end	architecture	user_component_mcp3201_arch;
